@@ -1,4 +1,3 @@
-static const char rcsid[]="$Id: handler.c,v 2.1 1999/01/02 09:35:07 roessler Exp $";
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
@@ -879,7 +878,7 @@ static void alternative_handler (BODY *a, STATE *s)
     while (b)
     {
       snprintf (buf, sizeof (buf), "%s/%s", TYPE (b), b->subtype);
-      if (mutt_is_autoview (buf))
+      if (mutt_is_autoview (b, buf))
       {
 	rfc1524_entry *entry = rfc1524_new_entry ();
 
@@ -977,7 +976,7 @@ void message_handler (BODY *a, STATE *s)
   if (b->parts)
   {
     mutt_copy_hdr (s->fpin, s->fpout, off_start, b->parts->offset,
-	(((s->flags & M_DISPLAY) && option (OPTWEED)) ? (CH_WEED | CH_REORDER) : 0) |
+	(((s->flags & M_WEED) || ((s->flags & M_DISPLAY) && option (OPTWEED))) ? (CH_WEED | CH_REORDER) : 0) |
 	(s->prefix ? CH_PREFIX : 0) | CH_DECODE | CH_FROM, s->prefix);
 
     if (s->prefix)
@@ -998,7 +997,7 @@ int mutt_can_decode (BODY *a)
   char type[STRING];
 
   snprintf (type, sizeof (type), "%s/%s", TYPE (a), a->subtype);
-  if (mutt_is_autoview (type))
+  if (mutt_is_autoview (a, type))
     return (rfc1524_mailcap_lookup (a, type, NULL, M_AUTOVIEW));
   else if (a->type == TYPETEXT)
     return (1);
@@ -1125,7 +1124,7 @@ void autoview_handler (BODY *a, STATE *s)
   rfc1524_mailcap_lookup (a, type, entry, M_AUTOVIEW);
 
   fname = safe_strdup (a->filename);
-  mutt_sanitize_filename (fname);
+  mutt_sanitize_filename (fname, 1);
   rfc1524_expand_filename (entry->nametemplate, fname, tempfile, sizeof (tempfile));
   FREE (&fname);
 
@@ -1261,7 +1260,7 @@ static void external_body_handler (BODY *b, STATE *s)
 		     CH_DECODE , NULL);
     }
   }
-  else if(expire < time(NULL))
+  else if(expiration && expire < time(NULL))
   {
     if (s->flags & M_DISPLAY)
     {
@@ -1323,7 +1322,7 @@ void mutt_body_handler (BODY *b, STATE *s)
   /* first determine which handler to use to process this part */
 
   snprintf (type, sizeof (type), "%s/%s", TYPE (b), b->subtype);
-  if (mutt_is_autoview (type))
+  if (mutt_is_autoview (b, type))
   {
     rfc1524_entry *entry = rfc1524_new_entry ();
 
