@@ -1,4 +1,3 @@
-static const char rcsid[]="$Id: send.c,v 2.1 1999/01/04 12:52:19 roessler Exp $";
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
@@ -316,8 +315,7 @@ static int include_forward (CONTEXT *ctx, HEADER *cur, FILE *out)
 {
   char buffer[STRING];
   int chflags = CH_DECODE, cmflags = 0;
-
-
+  
 
 #ifdef _PGPPATH
   if ((cur->pgp & PGPENCRYPT) && option (OPTFORWDECODE))
@@ -338,7 +336,10 @@ static int include_forward (CONTEXT *ctx, HEADER *cur, FILE *out)
   {
     cmflags |= M_CM_DECODE;
     if (option (OPTFORWWEEDHEADER))
+    {
       chflags |= CH_WEED;
+      cmflags |= M_CM_WEED;
+    }
   }
   if (option (OPTFORWQUOTE))
     cmflags |= M_CM_PREFIX;
@@ -1228,12 +1229,22 @@ main_loop:
   mutt_expand_path (fcc, sizeof (fcc));
   if (*fcc && mutt_strcmp ("/dev/null", fcc) != 0)
   {
+    struct stat st;
     BODY *tmpbody = msg->content;
 #ifdef _PGPPATH
     BODY *save_sig = NULL;
     BODY *save_parts = NULL;
 #endif /* _PGPPATH */
 
+    /* honor $confirmcreate and $confirmappend  in interactive mode */
+    if (!option (OPTNOCURSES) && !(flags & SENDMAILX) && 
+	!mutt_save_confirm (fcc, &st))
+    {
+      mutt_pretty_mailbox (fcc);
+      mutt_clear_error();
+      goto main_loop;
+    }
+
     /* check to see if the user wants copies of all attachments */
     if (!option (OPTFCCATTACH) && msg->content->type == TYPEMULTIPART)
     {
